<?php

declare(strict_types=1);

namespace Drupal\static_setting_contexts\Plugin\Condition;

use Drupal\Core\Condition\ConditionPluginBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\static_setting_contexts\StaticSettingsPluginManager;
use StaticSettings\InvalidStaticSettingsException;
use StaticSettings\StaticSettings as StaticSettingsPackage;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a 'Static Settings' condition.
 *
 * @Condition(
 *   id = "static_setting_contexts_static_settings",
 *   label = @Translation("Static Settings"),
 * )
 */
final class StaticSettings extends ConditionPluginBase implements ContainerFactoryPluginInterface {

  /**
   * The static settings definitions.
   *
   * @var array
   */
  private array $definitions;

  /**
   * Constructs a new StaticSettings instance.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    private readonly StaticSettingsPluginManager $pluginManagerStaticSettings,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->definitions = $this->pluginManagerStaticSettings->getDefinitions();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    return new self(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('plugin.manager.static_setting_contexts'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    $configuration = parent::defaultConfiguration();
    if (!isset($this->definitions)) {
      $this->definitions = $this->pluginManagerStaticSettings->getDefinitions();
    }
    array_walk($this->definitions, function ($definition, $id) use (&$configuration) {
      $configuration['static_settings'][$id] = [];
    });
    return $configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    array_walk($this->definitions, function ($definition, $id) use (&$form) {
      $class = $definition['class'];
      $cases = $class::cases();
      $keys = array_column($cases, 'name');
      $labels = array_map('ucwords', array_column($cases, 'value'));
      $options = array_combine($keys, $labels);
      $form['static_settings'][$id] = [
        '#type' => 'checkboxes',
        '#title' => $definition['label'],
        '#description' => $definition['description'],
        '#default_value' => $this->configuration['static_settings'][$id] ?? [],
        '#options' => $options,
      ];
    });
    return parent::buildConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state): void {
    array_walk($this->definitions, function ($definition, $id) use ($form_state) {
      $this->configuration['static_settings'][$id] = array_filter($form_state->getValue(['static_settings', $id]));
    });
    parent::submitConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function summary(): string {
    $settings = [];
    array_walk($this->definitions, function ($definition, $id) use (&$settings) {
      $values = array_filter($this->configuration['static_settings'][$id]);
      if (empty($values)) {
        return;
      }
      if ($this->isNegated()) {
        $settings[] = (string) $this->t('@label IS NOT @value', [
          '@label' => $definition['label'],
          '@value' => implode(' nor ', $values),
        ]);
      }
      else {
        $settings[] = (string) $this->t('@label IS @value', [
          '@label' => $definition['label'],
          '@value' => implode(' or ', $values),
        ]);
      }
    });
    return implode(' AND ', $settings);
  }

  /**
   * {@inheritdoc}
   */
  public function evaluate(): bool {
    if (empty($this->configuration['static_settings']) && !$this->isNegated()) {
      return TRUE;
    }

    $return = TRUE;

    array_walk($this->configuration['static_settings'], function ($values, $id) use (&$return) {
      try {
        $values = array_filter($values);
        if (empty($values)) {
          return;
        }
        $definition = $this->definitions[$id];
        $setting_value = StaticSettingsPackage::get($definition['class'])->name;
        if (!in_array($setting_value, $values)) {
          $return = FALSE;
        }
      }
      catch (InvalidStaticSettingsException) {
        // Do nothing.
      }

    });

    return $return;
  }

}
